@@ -1,6 +1,6 @@
 from .key import PublicKey, PrivateKey, bech32
 
-NIP19_PREFIXES = ('npub', 'nsec', 'note', 'nprofile', 'nevent', 'nrelay', 'nostr:')
+NIP19_PREFIXES = ('npub', 'nsec', 'note', 'nprofile', 'nevent', 'nrelay', 'nostr:', 'naddr')
 
 
 def from_nip19(nip19string: str):
@@ -21,8 +21,8 @@ def from_nip19(nip19string: str):
         retval['object'] = PrivateKey(bytes(data[:-1]))
     elif hrp == 'note':
         retval['object'] = bytes(data[:-1]).hex()
-    elif hrp in ('nevent', 'nprofile', 'nrelay'):
-        tlv = {0: [], 1: []}
+    elif hrp in ('nevent', 'nprofile', 'nrelay', 'naddr'):
+        tlv = {0: [], 1: [], 2: [], 3: []}
         while data:
             t = data[0]
             try:
@@ -35,7 +35,7 @@ def from_nip19(nip19string: str):
                 continue
             tlv[t].append(v)
         if tlv[0]:
-            if hrp != 'nrelay':
+            if hrp not in ('nrelay', 'naddr'):
                 key_or_id = bytes(tlv[0][0]).hex()
             else:
                 key_or_id = bytes(tlv[0][0]).decode()
@@ -44,37 +44,55 @@ def from_nip19(nip19string: str):
         relays = []
         for relay in tlv[1]:
             relays.append(bytes(relay).decode('utf8'))
+        if tlv[2]:
+            retval['author'] = bytes(tlv[2][0]).hex()
+        if tlv[3]:
+            retval['kind'] = int.from_bytes(bytes(tlv[3][0]), 'big')
         retval['object'] = key_or_id
         retval['relays'] = relays
     return retval
 
 
-def to_nip19(ntype: str, payload: str, relays=None):
+def to_nip19(ntype: str, payload: str, relays=None, author=None, kind=None):
     """
     Encode object as nip-19 compatible string
     """
     if ntype in ('npub', 'nsec', 'note'):
         data = bytes.fromhex(payload)
-    elif ntype in ('nprofile', 'nevent', 'nrelay'):
+    elif ntype in ('nprofile', 'nevent', 'nrelay', 'naddr'):
         data = bytearray()
         if ntype == 'nrelay':
-            # payload is the relay url
             encoded = payload.encode()
             data.append(0)
             data.append(len(encoded))
             data.extend(encoded)
+        elif ntype == 'naddr':
+            encoded = payload.encode()
+            data.append(0)
+            data.append(len(encoded))
+            data.extend(encoded)
+            if author:
+                author_encoded = bytes.fromhex(author)
+                data.append(2)
+                data.append(len(author_encoded))
+                data.extend(author_encoded)
+            if kind:
+                kind_bytes = kind.to_bytes(4, 'big')
+                data.append(3)
+                data.append(len(kind_bytes))
+                data.extend(kind_bytes)
         else:
             # payload is event id
             event_id = bytes.fromhex(payload)
             data.append(0)
             data.append(len(event_id))
             data.extend(event_id)
-            if relays:
-                for r in relays:
-                    r = r.encode()
-                    data.append(1)
-                    data.append(len(r))
-                    data.extend(r)
+        if relays:
+            for r in relays:
+                r = r.encode()
+                data.append(1)
+                data.append(len(r))
+                data.extend(r)
     else:
         data = payload.encode()
     converted_bits = bech32.convertbits(data, 8, 5)
